@@ -1,26 +1,3 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 /**
  * \file c-expr.c
  */
@@ -36,7 +13,6 @@
 #include "c-option.h"
 #include "c-pragma.h"
 
-
 CExpr                   *s_exprStart = NULL;
 CCOL_DList              s_symTabStack;
 CSymbolTable            *s_defaultSymTab;
@@ -1204,7 +1180,7 @@ innerCopyExprCommon(CExpr *dst, CExpr *src)
     CExprCommon *cdst = EXPR_C(dst);
     CExprCommon *csrc = EXPR_C(src);
 
-    memcpy(dst, src, sizeof(dst));
+    memcpy(dst, src, sizeof(CExpr));
     EXPR_C(dst)->e_refCount = 0;
 
     EXPR_SET0(cdst->e_gccAttrPre, copyExpr(csrc->e_gccAttrPre));
@@ -1321,101 +1297,103 @@ allocExprOfNumberConst(CExprCodeEnum exprCode, CBasicTypeEnum bt,
 
         errno = 0;
 
-        switch(bt) {
-        case BT_CHAR:
-        case BT_SHORT:
-        case BT_INT:
-        case BT_LONG:
-        case BT_LONGLONG: {
-                long long n = expr->e_numValue.ll = strtoll(digits, NULL, base);
-                if(errno) {
-                    overRange1 = 1;
-                    if(bt == BT_LONGLONG) {
-                        bt = BT_UNSIGNED_LONGLONG;
-                        errno = 0;
-                        n = expr->e_numValue.ull = strtoull(digits, NULL, base);
-                        overRange1 = (errno != 0);
-                    }
-                } else {
-                    switch(bt) {
-                    case BT_CHAR:
-                        if(n > UCHAR_MAX) overRange1 = 1;
-                        else if(n > SCHAR_MAX) bt = BT_UNSIGNED_CHAR;
-                        overRange1 = 1;
-                    case BT_SHORT:
-                        if(n > USHRT_MAX) overRange1 = 1;
-                        else if(n > SHRT_MAX) bt = BT_UNSIGNED_SHORT;
-                    case BT_INT:
-                        if(n > UINT_MAX) overRange1 = 1;
-                        else if(n > INT_MAX) bt = BT_UNSIGNED_INT;
-                    case BT_LONG:
-                        if(n > ULONG_MAX) overRange1 = 1;
-                        else if(n > LONG_MAX) bt = BT_UNSIGNED_LONG;
-                    default:
-                        break;
-                    }
-                }
-            }
+	if(strcmp(digits, "")){
+	  switch(bt) {
+	  case BT_CHAR:
+	  case BT_SHORT:
+	  case BT_INT:
+	  case BT_LONG:
+	  case BT_LONGLONG: {
+	    long long n = expr->e_numValue.ll = strtoll(digits, NULL, base);
+	    if(errno) {
+	      overRange1 = 1;
+	      if(bt == BT_LONGLONG) {
+		bt = BT_UNSIGNED_LONGLONG;
+		errno = 0;
+		n = expr->e_numValue.ull = strtoull(digits, NULL, base);
+		overRange1 = (errno != 0);
+	      }
+	    } else {
+	      switch(bt) {
+	      case BT_CHAR:
+		if(n > UCHAR_MAX) overRange1 = 1;
+		else if(n > SCHAR_MAX) bt = BT_UNSIGNED_CHAR;
+		overRange1 = 1;
+	      case BT_SHORT:
+		if(n > USHRT_MAX) overRange1 = 1;
+		else if(n > SHRT_MAX) bt = BT_UNSIGNED_SHORT;
+	      case BT_INT:
+		if(n > UINT_MAX) overRange1 = 1;
+		else if(n > INT_MAX) bt = BT_UNSIGNED_INT;
+	      case BT_LONG:
+		if(n > ULONG_MAX) overRange1 = 1;
+		else if(n > LONG_MAX) bt = BT_UNSIGNED_LONG;
+	      default:
+		break;
+	      }
+	    }
+	  }
             break;
-        case BT_UNSIGNED_CHAR:
-        case BT_UNSIGNED_SHORT:
-        case BT_UNSIGNED_INT:
-        case BT_UNSIGNED_LONG:
-        case BT_UNSIGNED_LONGLONG: {
-                unsigned long long n = expr->e_numValue.ull = strtoull(digits, NULL, base);
-                if(errno) {
-                    overRange1 = 1;
-                } else {
-                    switch(bt) {
-                    case BT_CHAR:
-                        overRange1 = (n > UCHAR_MAX); break;
-                    case BT_SHORT:
-                        overRange1 = (n > USHRT_MAX); break;
-                    case BT_INT:
-                        overRange1 = (n > UINT_MAX); break;
-                    case BT_LONG:
-                        overRange1 = (n > ULONG_MAX); break;
-                    default:
-                        break;
-                    }
-                }
-            }
+	  case BT_UNSIGNED_CHAR:
+	  case BT_UNSIGNED_SHORT:
+	  case BT_UNSIGNED_INT:
+	  case BT_UNSIGNED_LONG:
+	  case BT_UNSIGNED_LONGLONG: {
+	    unsigned long long n = expr->e_numValue.ull = strtoull(digits, NULL, base);
+	    if(errno) {
+	      overRange1 = 1;
+	    } else {
+	      switch(bt) {
+	      case BT_CHAR:
+		overRange1 = (n > UCHAR_MAX); break;
+	      case BT_SHORT:
+		overRange1 = (n > USHRT_MAX); break;
+	      case BT_INT:
+		overRange1 = (n > UINT_MAX); break;
+	      case BT_LONG:
+		overRange1 = (n > ULONG_MAX); break;
+	      default:
+		break;
+	      }
+	    }
+	  }
             break;
-        case BT_FLOAT:
-        case BT_FLOAT_IMAGINARY:
-          {
-            long double n = expr->e_numValue.ld = strtof(digits, NULL);
-            if(errno) {
-              overRange2 = 1;
-            }
-            else {
-              overRange2 = (n > FLT_MAX);
-            }
-          } break;
-        case BT_DOUBLE:
-        case BT_DOUBLE_IMAGINARY:
-          {
-            long double n = expr->e_numValue.ld = strtod(digits, NULL);
-            if(errno) {
-              overRange2 = 1;
-            }
-            else {
-              overRange2 = (n > DBL_MAX);
-            }
-          } break;
-        case BT_LONGDOUBLE:
-          {
-            long double n = expr->e_numValue.ld = strtold(digits, NULL);
-            if(errno) {
-              overRange2 = 1;
-            }
-            else {
-              overRange2 = (n > DBL_MAX);
-            }
-          } break;
-        default:
+	  case BT_FLOAT:
+	  case BT_FLOAT_IMAGINARY:
+	    {
+	      long double n = expr->e_numValue.ld = strtof(digits, NULL);
+	      if(errno) {
+		overRange2 = 1;
+	      }
+	      else {
+		overRange2 = (n > FLT_MAX);
+	      }
+	    } break;
+	  case BT_DOUBLE:
+	  case BT_DOUBLE_IMAGINARY:
+	    {
+	      long double n = expr->e_numValue.ld = strtod(digits, NULL);
+	      if(errno) {
+		overRange2 = 1;
+	      }
+	      else {
+		overRange2 = (n > DBL_MAX);
+	      }
+	    } break;
+	  case BT_LONGDOUBLE:
+	    {
+	      long double n = expr->e_numValue.ld = strtold(digits, NULL);
+	      if(errno) {
+		overRange2 = 1;
+	      }
+	      else {
+		overRange2 = (n > DBL_MAX);
+	      }
+	    } break;
+	  default:
             ABORT();
-        }
+	  }
+	}
     }
 
     expr->e_basicType = bt;
@@ -1857,6 +1835,35 @@ allocExprOfList4(CExprCodeEnum exprCode, CExpr *expr1, CExpr *expr2,
 }
 
 
+/**
+ * \brief
+ * alloc CExprOfList and 5 children
+ *
+ * @param exprCode
+ *      expression code
+ * @param expr1
+ *      child node
+ * @param expr2
+ *      child node
+ * @param expr3
+ *      child node
+ * @param expr4
+ *      child node
+ * @param expr5
+ *      child node
+ * @return
+ *      allocated node
+ */
+CExprOfList*
+allocExprOfList5(CExprCodeEnum exprCode, CExpr *expr1, CExpr *expr2,
+		 CExpr *expr3, CExpr *expr4, CExpr *expr5)
+{
+  CExprOfList *expr = allocExprOfList4(exprCode, expr1, expr2, expr3, expr4);
+  exprListAdd((CExpr*)expr, expr5);
+  return expr;
+}
+
+
 /**
  * \brief
  * free CExprOfList
