@@ -1,5 +1,5 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -1164,6 +1164,8 @@ extern CExprOfList*           allocExprOfList3(CExprCodeEnum exprCode,
                                     CExpr *e1, CExpr *e2, CExpr *e3);
 extern CExprOfList*           allocExprOfList4(CExprCodeEnum exprCode,
                                     CExpr *e1, CExpr *e2, CExpr *e3, CExpr *e4);
+extern CExprOfList*           allocExprOfList5(CExprCodeEnum exprCode,
+					       CExpr *e1, CExpr *e2, CExpr *e3, CExpr *e4, CExpr *e5);
 extern CExprOfNumberConst*    allocExprOfNumberConst(CExprCodeEnum exprCode,
                                     CBasicTypeEnum btype, CCardinalEnum c, char *orgToken);
 extern CExprOfNumberConst*    allocExprOfNumberConst1(CNumValueWithType *nvt);
