@@ -1,26 +1,3 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 /**
  * \file c-xcodeml.c
  */
@@ -348,10 +325,7 @@ outxChildren(FILE *fp, int indent, CExpr *expr)
     CExprIterator ite;
     EXPR_FOREACH_MULTI(ite, expr)
         if(ite.node) {
-            if(EXPR_CODE(ite.node) == EC_NULL_NODE)   // ID=284
-                outxPrint(fp, indent, "<list/>\n");
-            else
-                outxContext(fp, indent, ite.node);
+            outxContext(fp, indent, ite.node);
         }
 }
 
@@ -1298,7 +1272,7 @@ void
 setTypeIds()
 {
     CCOL_DListNode *ite;
-
+    
     CCOL_DL_FOREACH(ite, &s_typeDescList) {
         (EXPR_T(CCOL_DL_DATA(ite)))->e_isMarked = 0;
     }
