@@ -1,5 +1,5 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -82,6 +82,7 @@ static CExpr* parse_POST_clause();
 static CExpr* parse_WAIT_clause();
 //static CExpr* parse_LOCAL_ALIAS_clause();
 static CExpr* parse_WIDTH_list();
+static CExpr* parse_ASYNC_clause();
 static CExpr* parse_WAIT_ASYNC_clause();
 static CExpr* parse_TEMPLATE_FIX_clause();
 
@@ -105,6 +106,7 @@ static CExpr* _xmp_pg_list(int omp_code,CExpr* args);
 #define XMP_LIST2(arg1,arg2) (CExpr*)allocExprOfList2(EC_UNDEF,arg1,arg2)
 #define XMP_LIST3(arg1,arg2,arg3) (CExpr*)allocExprOfList3(EC_UNDEF,arg1,arg2,arg3)
 #define XMP_LIST4(arg1,arg2,arg3,arg4) (CExpr*)allocExprOfList4(EC_UNDEF,arg1,arg2,arg3,arg4)
+#define XMP_LIST5(arg1,arg2,arg3,arg4,arg5) (CExpr*)allocExprOfList5(EC_UNDEF,arg1,arg2,arg3,arg4,arg5)
 
 #define XMP_Error0(msg) addError(NULL,msg)
 #define XMP_error1(msg,arg1) addError(NULL,msg,arg1)
@@ -1392,7 +1394,7 @@ CExpr *parse_XMP_opt()
   return (CExpr *)allocExprOfNull();
 }
 
-CExpr* parse_TASKS_clause()
+CExpr *parse_TASKS_clause()
 {
     return (CExpr *)allocExprOfNull();
 }
@@ -1401,16 +1403,17 @@ static CExpr* parse_REFLECT_clause()
 {
     CExpr *arrayNameList = parse_name_list();
     CExpr *widthList = parse_WIDTH_list();
-    CExpr *async = (CExpr *)allocExprOfNull();
+    CExpr *async = parse_ASYNC_clause();
+    /* CExpr *async = (CExpr *)allocExprOfNull(); */
 
-    if (PG_IS_IDENT("async")){
-      pg_get_token();
-      if (pg_tok != '(') goto err;
-      pg_get_token();
-      async = pg_parse_expr();
-      if (pg_tok != ')') goto err;
-      pg_get_token();
-    }
+    /* if (PG_IS_IDENT("async")){ */
+    /*   pg_get_token(); */
+    /*   if (pg_tok != '(') goto err; */
+    /*   pg_get_token(); */
+    /*   async = pg_parse_expr(); */
+    /*   if (pg_tok != ')') goto err; */
+    /*   pg_get_token(); */
+    /* } */
 
     CExpr *profileClause = (CExpr *)allocExprOfNull();
     /* if (pg_is_ident("profile")) { */
@@ -1419,11 +1422,12 @@ static CExpr* parse_REFLECT_clause()
     /* } */
 
     return XMP_LIST4(arrayNameList, widthList, async, profileClause);
-
+/*
  err:
     XMP_Error0("syntax error in the REFLECT directive");
     XMP_has_err = 1;
     return NULL;
+*/
 }
 
 
@@ -1431,6 +1435,7 @@ static CExpr* parse_REDUCTION_clause()
 {
     CExpr* reductionRef = parse_Reduction_ref();
     CExpr* onRef = (CExpr *)allocExprOfNull();
+    CExpr* async;
     CExpr* acc_or_host = (CExpr *)allocExprOfNull();
 
     if (PG_IS_IDENT("on")) {
@@ -1438,6 +1443,8 @@ static CExpr* parse_REDUCTION_clause()
       onRef = parse_task_ON_ref();
     }
 
+    async = parse_ASYNC_clause();
+
     if(PG_IS_IDENT("acc") || PG_IS_IDENT("host")){
       acc_or_host = XMP_LIST1(pg_tok_val);
       pg_get_token();
@@ -1453,7 +1460,7 @@ static CExpr* parse_REDUCTION_clause()
     /* 	    pg_get_token(); */
     /* 	} */
 
-    return XMP_LIST4(reductionRef, onRef, acc_or_host, profileClause);
+    return XMP_LIST5(reductionRef, onRef, async, acc_or_host, profileClause);
 }
 
 static CExpr* parse_BARRIER_clause()
@@ -1493,13 +1500,15 @@ static CExpr* parse_BCAST_clause()
     }
     else onRef = (CExpr *)allocExprOfNull();;
 
+    CExpr* async = parse_ASYNC_clause();
+
     CExpr* profileClause = (CExpr *)allocExprOfNull();;
     /* if (PG_IS_IDENT("profile")) { */
     /*     profileClause = Xcons.StringConstant("profile"); */
     /*     pg_get_token(); */
     /* } */
 
-    return XMP_LIST4(varList, fromRef, onRef, profileClause);
+    return XMP_LIST5(varList, fromRef, onRef, async, profileClause);
 }
 
 static CExpr* parse_GMOVE_clause()
@@ -1777,10 +1786,37 @@ static CExpr* parse_WIDTH_list()
 }
 
 
+static CExpr* parse_ASYNC_clause()
+{
+  CExpr *async = (CExpr *)allocExprOfNull();
+
+  if (PG_IS_IDENT("async")){
+    pg_get_token();
+    if (pg_tok != '(') goto err;
+    pg_get_token();
+    async = pg_parse_expr();
+    if (pg_tok != ')') goto err;
+    pg_get_token();
+  }
+
+  return async;
+
+ err:
+    XMP_Error0("syntax error in the REFLECT directive");
+    XMP_has_err = 1;
+    return NULL;
+
+}
+
 static CExpr* parse_WAIT_ASYNC_clause()
 {
     CExpr *asyncIdList = parse_expr_list();
-    return XMP_LIST1(asyncIdList);
+    CExpr* onRef = NULL;
+    if (PG_IS_IDENT("on")){
+      pg_get_token();
+      onRef = parse_task_ON_ref();
+    }
+    return XMP_LIST2(asyncIdList, onRef);
 }
 
 
