@@ -1,5 +1,5 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -267,8 +267,13 @@ void compile_XMP_directive(expr x)
 
     case XMP_ARRAY:
       check_INEXEC();
+      /* check arg: node_ref opt */
+      x1 = XMP_compile_ON_ref(EXPR_ARG1(c));
+      x2 = XMP_compile_clause_opt(EXPR_ARG2(c));
+      c = list2(LIST,x1,x2);
       XMP_st_required = XMP_ST_ARRAY;
-      XMP_array_directive = x;
+      //XMP_array_directive = x;
+      XMP_array_directive = list2(XMP_PRAGMA, EXPR_ARG1(x), list2(LIST, x1, x2));
       break;
 
     case XMP_REFLECT:
@@ -298,15 +303,18 @@ void compile_XMP_directive(expr x)
 
     case XMP_WAIT_ASYNC:
       check_INEXEC();
-      expr c0 = list0(LIST);
-      list lp;
-      FOR_ITEMS_IN_LIST(lp, EXPR_ARG1(c)){
-        x1 = LIST_ITEM(lp);
-	x1 = compile_expression(x1);
-	c0 = list_put_last(c0, x1);
-      }
-      c = list1(LIST,c0);
-      output_statement(XMP_pragma_list(XMP_WAIT_ASYNC,c,NULL));
+      output_statement(x);
+      /* x2 = XMP_compile_ON_ref(EXPR_ARG2(c)); */
+      /* expr c0 = list0(LIST); */
+      /* list lp; */
+      /* FOR_ITEMS_IN_LIST(lp, EXPR_ARG1(c)){ */
+      /*   x1 = LIST_ITEM(lp); */
+      /* 	x1 = compile_expression(x1); */
+      /* 	c0 = list_put_last(c0, x1); */
+      /* } */
+      /* c = list1(LIST,c0); */
+      /* //output_statement(XMP_pragma_list(XMP_WAIT_ASYNC,c,NULL)); */
+      /* output_statement(XMP_pragma_list(XMP_WAIT_ASYNC,c, x2)); */
       break;
 
     case XMP_TEMPLATE_FIX:
@@ -592,8 +600,8 @@ expv XMP_compile_subscript_list(expr l,xmp_list_context context)
 		if(EXPR_ARG1(x) == EXPR_ARG2(x) &&
 		   EXPR_ARG3(x) != NULL && EXPR_INT(EXPR_ARG3(x)) == 0){
 		    v = compile_expression(EXPR_ARG1(x));
-		    if(!IS_INT_CONST_V(v) && !IS_INT_PARAM_V(v))
-			error("subscript in nodes must be an integer constant");
+		    /* if(!IS_INT_CONST_V(v) && !IS_INT_PARAM_V(v)) */
+		    /* 	error("subscript in nodes must be an integer constant"); */
 		} else 
 		    error("bad subscript in nodes directive");
 	    }
