@@ -1,5 +1,5 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -1688,6 +1688,12 @@ compile_array_ref(ID id, expv vary, expr args, int isLeft) {
         argASpec = expr_list_get_n(aSpecs, i);
         if (argASpec != NULL) {
             idASpec = expr_list_get_n(idShape, i);
+
+            expv lower = expr_list_get_n(argASpec, 0);
+            if (lower == NULL) expr_list_set_n(argASpec, 0, expr_list_get_n(idASpec, 0), FALSE);
+	    expv upper = expr_list_get_n(argASpec, 1);
+            if (upper == NULL) expr_list_set_n(argASpec, 1, expr_list_get_n(idASpec, 1), FALSE);
+
             /*
              * Now we have two array-spec. Determine which one to be used.
              */
