@@ -1,5 +1,5 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -197,9 +197,15 @@ public class Xcons
         return new XobjList(Xcode.ID_LIST);
     }
 
-    public static Ident Ident(String name, StorageClass stg_class, Xtype type, Xobject v, VarScope scope)
+    public static Ident Ident(String name, StorageClass stg_class, Xtype type,
+                              Xobject v, VarScope scope)
     {
-        return new Ident(name, stg_class, type, v, scope);
+        return new Ident(name, stg_class, type, v, scope, null);
+    }
+    public static Ident Ident(String name, StorageClass stg_class, Xtype type,
+                              Xobject v, VarScope scope, Xobject codimensions)
+    {
+        return new Ident(name, stg_class, type, v, scope, codimensions);
     }
     
     //
@@ -400,7 +406,8 @@ public class Xcons
                 t = ConversionIntegral(lt, rt);
                 break;
             }
-            if(lt.isFloating() && rt.isFloating()) {
+            //if(lt.isFloating() && rt.isFloating()) {
+            if(lt.isNumeric() && rt.isNumeric()) {           // #357
                 t = BasicType.Conversion(lt, rt);
                 break;
             }
@@ -429,7 +436,8 @@ public class Xcons
                 t = ConversionIntegral(lt, rt);
                 break;
             }
-            if(lt.isFloating() && rt.isFloating()) {
+            //if(lt.isFloating() && rt.isFloating()) {
+            if(lt.isNumeric() && rt.isNumeric()) {           // #357
                 t = BasicType.Conversion(lt, rt);
                 break;
             }
@@ -442,6 +450,10 @@ public class Xcons
                 t = ConversionIntegral(lt, rt);
                 break;
             }
+            if(lt.isNumeric() && rt.isNumeric()) {           // #357
+                t = BasicType.Conversion(lt, rt);
+                break;
+            }
             fatal("BinaryOp: bad type");
 
         case BIT_OR_EXPR:
