@@ -1,5 +1,5 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -40,4 +40,10 @@ public abstract class XobjConst extends Xobject
     {
         return fkind;
     }
+
+    @Override
+    public int getFrank()
+    {
+        return 0;
+    }
 }
