@@ -1,5 +1,5 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -23,12 +23,14 @@
  */
 package exc.object;
 
+import exc.block.Block;
 import exc.util.XobjectVisitable;
 import exc.util.XobjectVisitor;
 import xcodeml.IXobject;
 import xcodeml.XmException;
 import xcodeml.util.XmOption;
 
+
 /**
  * Abstract class of expression tree objects in Xobject.
  */
@@ -343,6 +345,17 @@ public class Xobject extends PropObject implements IXobject, XobjectVisitable, I
         throw new UnsupportedOperationException(toString());
     }
 
+    /**
+     * Get rank in the term of Fortran.
+     * Eg: Rank of a scalar expr is 0.
+     *     Rank of a subarray is equal to or less than the rank of
+     *     the host array.
+     */
+    public int getFrank()
+    {
+        throw new UnsupportedOperationException(toString());
+    }
+
     /**
      * Return argument list in XobjArgs.
      * Dummy method at base class, it causes exception.
@@ -539,6 +552,7 @@ public class Xobject extends PropObject implements IXobject, XobjectVisitable, I
         return o;
     }
 
+
     /**
      * 
      * Dummy method at base class, it causes exception.
@@ -866,6 +880,12 @@ public class Xobject extends PropObject implements IXobject, XobjectVisitable, I
         return null;
     }
     
+    public Xobject cfold(Block block)
+    {
+      /* default */
+      return this.copy();
+    }
+
     public Ident findVarIdent(String name)
     {
         return find(name, IXobject.FINDKIND_VAR);
