@@ -1,5 +1,5 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -50,6 +50,8 @@ public class XobjectFile extends XobjectDefEnv
     public boolean debugFlag;
     public static boolean gcc_huge_common_bug;
     
+    private String tailText = "";     // for collectInit temporary version
+
     /** default constructor */
     public XobjectFile()
     {
@@ -119,7 +121,7 @@ public class XobjectFile extends XobjectDefEnv
         addrType = Xtype.Pointer(t);
         
         return new Ident(name, sclass, t,
-            Xcons.Symbol(addrCode, addrType, name), VarScope.GLOBAL);
+                         Xcons.Symbol(addrCode, addrType, name), VarScope.GLOBAL, null);
     }
 
     /**
@@ -346,7 +348,7 @@ public class XobjectFile extends XobjectDefEnv
         if(type.isFintentIN())      out.print(" fintentIN");
         if(type.isFintentOUT())     out.print(" fintentOUT");
         if(type.isFintentINOUT())   out.print(" fintentINOUT");
-        if(type.isFcrayPointer())   out.print(" fcrayPointer");  // (ID=60)
+        if(type.isFcrayPointer())   out.print(" fcrayPointer");  // (ID=060c)
         if(type.isFprogram())       out.print(" fprogram");
         if(type.isFintrinsic())     out.print(" fintrinsic");
         if(type.isFrecursive())     out.print(" frecursive");
@@ -557,4 +559,19 @@ public class XobjectFile extends XobjectDefEnv
             collectType(i.getGccAttributes());
         }
     }
+
+
+    /*
+     *  handling Tail Text -- for collect init
+     */
+    public String getTailText() {
+        return tailText;
+    }
+    public void clearTailText() {
+        tailText = "";
+    }
+    public void addTailText(String text) {
+        tailText += text;
+    }
+
 }
