@@ -1,5 +1,5 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -27,6 +27,7 @@ import exc.block.Block;
 import exc.util.MachineDepConst;
 
 import xcodeml.util.XmOption;
+import static xcodeml.util.XmLog.fatal;
 
 public class Xtype
 {
@@ -41,6 +42,7 @@ public class Xtype
     public final static int POINTER         = 7;
     public final static int F_ARRAY         = 8;
     public final static int XMP_CO_ARRAY    = 9;
+    public final static int F_COARRAY       = 10;        // ID=060
 
     final static String kind_names[] = {
         "UNDEF",
@@ -53,6 +55,7 @@ public class Xtype
         "POINTER",
         "F_ARRAY",
         "XMP_COARRAY",
+        "F_COARRAY",        // ID=060
     };
     
     //
@@ -86,7 +89,7 @@ public class Xtype
     public static final int TQ_FEXTERNAL            = 1 << 21;  // external
     public static final int TQ_FSEQUENCE            = 1 << 22;  // sequence
     public static final int TQ_FINTERNAL_PRIVATE    = 1 << 23;  // private in structure decl
-    public static final int TQ_FCRAY_POINTER        = 1 << 24;  // cray pointer (ID=60)
+    public static final int TQ_FCRAY_POINTER        = 1 << 24;  // cray pointer (ID=060c)
     
     private String type_id;
     private int type_kind;
@@ -109,6 +112,10 @@ public class Xtype
     /** type name */
     protected Ident tag;
     
+    /** coshape infos. incl. corank and codimensions (ID=060) */
+    protected boolean is_coarray = false;
+    protected Coshape coshape = new Coshape();
+
     /*
      * for pre-defined basic type
      */
@@ -241,23 +248,37 @@ public class Xtype
     };
 
     // constructor
-    public Xtype(int kind, String id, int typeQualFlags, Xobject gccAttrs, Ident tag)
+    public Xtype(int kind, String id, int typeQualFlags, Xobject gccAttrs,
+                 Ident tag, Xobject[] codimensions)
     {
         this.type_kind = kind;
         this.type_id = id;
         setTypeQualFlags(typeQualFlags);
         this.gcc_attrs = gccAttrs;
         this.tag = tag;
+        setCodimensions(codimensions);
+    }
+
+    public Xtype(int kind, String id, int typeQualFlags, Xobject gccAttrs,
+                 Ident tag)
+    {
+        this(kind, id, typeQualFlags, gccAttrs, tag, null);
+    }
+    
+    public Xtype(int kind, String id, int typeQualFlags, Xobject gccAttrs,
+                 Xobject[] codimensions)
+    {
+        this(kind, id, typeQualFlags, gccAttrs, null, codimensions);
     }
     
     public Xtype(int kind, String id, int typeQualFlags, Xobject gccAttrs)
     {
-    	this(kind, id, typeQualFlags, gccAttrs, null);
+        this(kind, id, typeQualFlags, gccAttrs, null, null);
     }
     
     public Xtype(int kind)
     {
-        this(kind, null, 0, null, null);
+        this(kind, null, 0, null, null, null);
     }
 
     /** return if is qualifed by qualifier,
@@ -494,13 +515,13 @@ public class Xtype
         setTypeQualFlag(TQ_FALLOCATABLE, enabled);
     }
 
-    /** Fortran : return if it is a cray pointer (ID=60) */
+    /** Fortran : return if it is a cray pointer (ID=060c) */
     public final boolean isFcrayPointer()
     {
         return getTypeQualFlag(TQ_FCRAY_POINTER);
     }
     
-    /** Fortran : set qualifier 'cray pointer' (ID=60) */
+    /** Fortran : set qualifier 'cray pointer' (ID=060c) */
     public final void setIsFcrayPointer(boolean enabled)
     {
         setTypeQualFlag(TQ_FCRAY_POINTER, enabled);
@@ -620,7 +641,7 @@ public class Xtype
         return getTypeQualFlag(TQ_FINTERNAL_PRIVATE);
     }
     
-    /** Fortran : set qualifier 'private ' in structure decl */
+    /** Fortran : set qualifier 'private' in structure decl */
     public final void setIsFinternalPrivate(boolean enabled)
     {
         setTypeQualFlag(TQ_FINTERNAL_PRIVATE, enabled);
@@ -661,6 +682,12 @@ public class Xtype
     {
         throw new UnsupportedOperationException();
     }
+
+    /** Fortran: get function result name */
+    public void setFuncResultName(String fresult_name)
+    {
+        throw new UnsupportedOperationException();
+    }
     
     @Deprecated
     public final long getArrayDim()
@@ -704,6 +731,24 @@ public class Xtype
         throw new UnsupportedOperationException();
     }
 
+    /** Fortran: get Fortran array size or 1 for scalar */     // #060
+    public Xobject getTotalArraySizeExpr(Block block)
+    {
+        throw new UnsupportedOperationException();
+    }
+
+    /** Fortran: get Fortran type element length (bytes) in Expr */     // #060
+    public Xobject getElementLengthExpr(Block block)
+    {
+        throw new UnsupportedOperationException();
+    }
+
+    /** Fortran: get Fortran type element length (bytes) in integer */     // #060
+    public int getElementLength(Block block)
+    {
+        throw new UnsupportedOperationException();
+    }
+
     /** Fortran: return if is assumed size array */
     public boolean isFassumedSize()
     {
@@ -746,6 +791,48 @@ public class Xtype
         throw new UnsupportedOperationException();
     }
 
+    /*
+     *  implements Coshape (#060)
+     */
+    public int getCorank()
+    {
+        return coshape.getCorank();
+    }
+    public boolean isCoarray()
+    {
+        return is_coarray;
+    }
+    public void setIsCoarray(boolean is_coarray)
+    {
+        this.is_coarray = is_coarray;
+    }
+    public boolean wasCoarray()
+    {
+        return getCorank() > 0;
+    }
+    public Xobject[] getCodimensions()
+    {
+        return coshape.getCodimensions();
+    }
+    public void setCodimensions(Xobject[] codimensions)
+    {
+        coshape.setCodimensions(codimensions);
+        is_coarray  = (getCorank() > 0);
+    }
+    public void removeCodimensions()
+    {
+        coshape.removeCodimensions();
+        is_coarray  = false;
+    }
+    public void hideCodimensions()
+    {
+        is_coarray  = false;
+    }
+     public Xobject[] copyCodimensions()
+    {
+        return coshape.copyCodimensions();
+    }
+
     /** @deprecated */
     @Deprecated
     public XobjList getMoeList()
@@ -774,7 +861,7 @@ public class Xtype
     /** create copy */
     protected Xtype copy(String id)
     {
-        return new Xtype(type_kind, id, getTypeQualFlags(), gcc_attrs, tag);
+        return new Xtype(type_kind, id, getTypeQualFlags(), gcc_attrs, tag, copyCodimensions());
     }
     
     /** create copy. created copy references this instance as copied */
@@ -876,7 +963,8 @@ public class Xtype
     }
 
     /** return if is numeric type */
-    public final boolean isNumeric()
+    //public final boolean isNumeric()       #357
+    public boolean isNumeric()
     {
         return isIntegral() || isFloating() || isComplexOrImaginary();
     }
@@ -899,7 +987,7 @@ public class Xtype
         return null;
     }
     
-    /** Fortran: return if len parameter is varaible value */
+    /** Fortran: return if len parameter is variable value */
     public boolean isFlenVariable()
     {
         return false;
