@@ -1,6 +1,6 @@
 package exc.openacc;
 /*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -165,8 +165,10 @@ public class ACCgpuDecompileWriter extends PrintWriter {
             printDeclList(v.getArg(2), v.getArg(1));
           }
 
+          print("{");
           printBody(v.getArg(3));
           println();
+          println("}");
         } break;
       default:
         fatal("wrong operation");
@@ -471,17 +473,6 @@ public class ACCgpuDecompileWriter extends PrintWriter {
       if (prop != null) {
         println(";");
 
-        if(propAsync == null){
-          println("_ACC_GPU_M_BARRIER_KERNEL();");
-          if(ACC.debugFlag){
-            println("_ACC_GPU_CHECK_ERROR(\"" + funcName + "\");"); //for debugging
-          }
-        }else if(!propAsync.isEmpty()){
-          //_ACC_gpu_sync_kernel(id)のような関数呼び出しにする
-          print("if(");
-          print(propAsync.getArg(0));
-          println(" == ACC_ASYNC_SYNC) _ACC_gpu_wait(ACC_ASYNC_SYNC);");
-        }
         print("}");
       }
       break;
@@ -496,6 +487,17 @@ public class ACCgpuDecompileWriter extends PrintWriter {
       print(v.left());
       print(")--");
       break;
+      
+    case PRE_INCR_EXPR:
+      print("++(");
+      print(v.left());
+      print(")");
+      break;
+    case PRE_DECR_EXPR:
+      print("--(");
+      print(v.left());
+      print(")");
+      break;
 
     case SIZE_OF_EXPR:    /* (SIZE_OF_EXPR type-or-expr) */
       print("sizeof(");
@@ -768,7 +770,7 @@ public class ACCgpuDecompileWriter extends PrintWriter {
   private String makeDeclType(Xtype type, String name) {
     String decltype = "";
     Xtype t;
-    Stack nested_decls = new Stack();
+    Stack<Xtype> nested_decls = new Stack<Xtype>();
 
     for (t = type; t != null; t = t.getRef()) {
       if (t.getKind() == Xtype.POINTER ||
@@ -881,7 +883,7 @@ public class ACCgpuDecompileWriter extends PrintWriter {
         if (t.getArraySize() <= 0) {
           decltype += "[]";
         } else {
-          decltype += "[" + t.getArrayDim() + "]";
+          decltype += "[" + t.getArraySize() + "]";
         }
       } break;
     default:
@@ -1058,14 +1060,14 @@ public class ACCgpuDecompileWriter extends PrintWriter {
           } break;
         case Xtype.ENUM:
           {
-            if(type.getMoeList() == null) {
+            if(type.getMemberList() == null) {
               break;
             }
 
             printTypeName(type);
             println(" {");
 
-            for (XobjArgs a1 = type.getMoeList().getArgs(); a1 != null; a1=a1.nextArgs()) {
+            for (XobjArgs a1 = type.getMemberList().getArgs(); a1 != null; a1=a1.nextArgs()) {
               if ((id = findIdent(id_list, a1.getArg())) == null) {
                 fatal("Enum ID is not found");
               }
