@@ -1,5 +1,5 @@
 /*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -450,6 +450,7 @@ public class XMPalignedArray {
     if (isLocalPragma) {
       if (!isPointer) arrayAddrId = XMPlocalDecl.addObjectId2(XMP.ADDR_PREFIX_ + arrayName,
 							      Xtype.Pointer(arrayElmtType), parentBlock);
+      else arrayAddrId.setType(Xtype.Pointer(arrayElmtType));
       arrayDescId = XMPlocalDecl.addObjectId2(XMP.DESC_PREFIX_ + arrayName, parentBlock);
     }
     else {
@@ -467,6 +468,7 @@ public class XMPalignedArray {
 	  throw new XMPexception("cannot align array '" + arrayName + "', wrong storage class");
 	}
       }
+      else arrayAddrId.setType(Xtype.Pointer(arrayElmtType));
       arrayDescId = globalDecl.declStaticIdent(XMP.DESC_PREFIX_ + arrayName, Xtype.voidPtrType);
     }
 
@@ -814,7 +816,7 @@ public class XMPalignedArray {
 
     // assumed that dynamic arrays are one-dimentional.
     alignFuncArgs.add(Xcons.Cast(Xtype.Pointer(Xtype.unsignedlonglongType),
-				 alignedArray.getAccIdAt(0).getAddr()));
+				 alignedArray.getAccIdAt(alignSourceIndex).getAddr()));
 
     if (isLocalPragma) {
       XMPlocalDecl.addConstructorCall2("_XMP_align_array_noalloc", alignFuncArgs, globalDecl, parentBlock);
