@@ -1,5 +1,5 @@
 /*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -46,6 +46,7 @@ public class XMPtemplate extends XMPobject {
   private Vector<Xobject>       _widthVector;
   private XobjList              _decl;
   private XobjList              _distDecl;
+  private Vector<Ident>		_gtolTemp0IdVector;
 
   public XMPtemplate(String name, int dim, Ident descId) {
     super(XMPobject.TEMPLATE, name, dim, descId);
@@ -58,11 +59,13 @@ public class XMPtemplate extends XMPobject {
     _distMannerVector = new Vector<Integer>();
     _sizeVector = new Vector<Xobject>();
     _widthVector = new Vector<Xobject>();
+    _gtolTemp0IdVector = new Vector<Ident>();
 
     for (int i = 0; i < dim; i++) {
       _ontoNodesIndexVector.add(null);
       _distMannerVector.add(null);
       _widthVector.add(null);
+      _gtolTemp0IdVector.add(null);
     }
     
     _decl = null;
@@ -188,6 +191,14 @@ public class XMPtemplate extends XMPobject {
     return _distDecl;
   }
 
+  public void setGtolTemp0IdAt(Ident temp0Id, int index) {
+    _gtolTemp0IdVector.setElementAt(temp0Id, index);
+  }
+
+  public Ident getGtolTemp0IdAt(int index) {
+    return _gtolTemp0IdVector.get(index);
+  }
+
   @Override
   public boolean checkInheritExec() {
     return _ontoNodes.checkInheritExec();
@@ -302,19 +313,19 @@ public class XMPtemplate extends XMPobject {
     // get template object
     String templateName = distDecl.getArg(0).getString();
     XMPtemplate templateObject = null;
-    if (isLocalPragma) {
-      templateObject = localXMPsymbolTable.getXMPtemplate(templateName);
-      if (templateObject == null) {
-        templateObject = globalDecl.getXMPtemplate(templateName);
-        if (templateObject != null) {
-          throw new XMPexception("global template cannot be distributed in local scope");
-        }
-      }
-    }
-    else {
-      templateObject = globalDecl.getXMPtemplate(templateName);
-    }
-
+    // if (isLocalPragma) {
+    //   templateObject = localXMPsymbolTable.getXMPtemplate(templateName);
+    //   if (templateObject == null) {
+    //     templateObject = globalDecl.getXMPtemplate(templateName);
+    //     if (templateObject != null) {
+    //       throw new XMPexception("global template cannot be distributed in local scope");
+    //     }
+    //   }
+    // }
+    // else {
+    //   templateObject = globalDecl.getXMPtemplate(templateName);
+    // }
+    templateObject = globalDecl.getXMPtemplate(templateName, pb);
     if (templateObject == null) {
       throw new XMPexception("template '" + templateName + "' is not declared");
     }
@@ -453,10 +464,26 @@ public class XMPtemplate extends XMPobject {
 	    mappingArrayArg = Xcons.Cast(Xtype.voidPtrType, Xcons.IntConstant(0));
 	    templateObject.setDistributionIsFixed(false);
 	  }
-          funcArgs = Xcons.List(templateObject.getDescId().Ref(),
-                                Xcons.IntConstant(templateDimIdx),
-                                Xcons.IntConstant(nodesDimIdx),
-                                mappingArrayArg);
+
+	  if (templateObject.isFixed() || isTFIX){
+	    Ident gtolTemp0Id = null;
+	    String tempName = XMP.GTOL_PREFIX_ + "temp0_" + templateObject.getName() + "_" + templateDimIdx;
+	    if (isLocalPragma) {
+	      Block parentBlock = pb.getParentBlock();
+	      gtolTemp0Id = XMPlocalDecl.addObjectId2(tempName, Xtype.intType, parentBlock);
+	    }
+	    else {
+	      gtolTemp0Id = globalDecl.declStaticIdent(tempName, Xtype.intType);
+	    }
+
+	    templateObject.setGtolTemp0IdAt(gtolTemp0Id, templateDimIdx);
+
+	    funcArgs = Xcons.List(templateObject.getDescId().Ref(),
+				  Xcons.IntConstant(templateDimIdx),
+				  Xcons.IntConstant(nodesDimIdx),
+				  mappingArrayArg, gtolTemp0Id.getAddr());
+	  }
+
           templateObject.setOntoNodesIndexAt(nodesDimIdx, templateDimIdx);
 	  templateObject.setWidthAt(mappingArray, templateDimIdx);
           break;
