@@ -527,7 +527,7 @@ public class XcodeMLtools_C extends XcodeMLtools {
     // get enum member value
     Xobject enumValue = toXobject(getContent(getElement(n, "value")));
 
-    // get codimensions ID=284
+    // get codimensions #284
     Xobject codims = toXobject(getElement(n, "codimensions"));
 
     // get gcc attributes
@@ -563,12 +563,12 @@ public class XcodeMLtools_C extends XcodeMLtools {
     }
 
     // create ident
+    // for coarray, set codiemnsions (#284)
     Ident ident = new Ident(name, sclass, type, addr,
 			    optionalFlags, gccAttrs,
-			    bitField, bitFieldExpr, enumValue, null);
-    // for coarray, set codiemnsions (ID=284)
-    if (codims != null)
-        ident.setCodimensions(codims);
+			    bitField, bitFieldExpr, enumValue, null, codims);
+    //if (codims != null)
+    //  ident.setCodimensions(codims);
 
     // declaring
     if (sclass != null && sclass.isVarOrFunc()) {
