@@ -1,5 +1,5 @@
 /*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -122,6 +122,18 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
                                         trans(ident.getBitFieldExpr())));
         }
 
+        // gccThread
+        if (ident.isGccThread()) {
+            addAttributes(e,
+                          "is_gccThread", "1");
+        }
+
+        // gccExtension
+        if (ident.isGccExtension()) {
+            addAttributes(e,
+                          "is_gccExtension", "1");
+        }
+
         // enum member value
         addChildNodes(e,
                       transValue(ident.getEnumValue()));
