@@ -1,5 +1,5 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -592,15 +592,21 @@ public class XMPanalyzePragma
 			XMPinfo info, PragmaBlock pb){
     Xobject reductionSpec = reductionDecl.getArg(0);
     Xobject reductionOnRef = reductionDecl.getArg(1);
-    Xobject reductionOpt = reductionDecl.getArg(2);
+    Xobject asyncOpt = reductionDecl.getArg(2);
 
-    if(reductionOpt != null){
-      XMP.fatal("redution opt is not supported yet, sorry!");
-      return;
-    }
+    // if(reductionOpt != null){
+    //   XMP.fatal("redution opt is not supported yet, sorry!");
+    //   return;
+    // }
 
     analyzeReductionSpec(info, reductionSpec, pb);
     info.setOnRef(XMPobjectsRef.parseDecl(reductionOnRef,env,pb));
+
+    if (asyncOpt != null && !XmOption.isAsync()){
+      XMP.errorAt(pb, "MPI-3 is required to use the async clause on a reduction directive");
+    }
+
+    info.setAsyncId(asyncOpt);
   }
 
   private void analyzeReductionSpec(XMPinfo info, Xobject reductionSpec,
@@ -645,12 +651,12 @@ public class XMPanalyzePragma
     XobjList bcastNameList = (XobjList) bcastDecl.getArg(0);
     Xobject fromRef = bcastDecl.getArg(1);
     Xobject onRef = bcastDecl.getArg(2);
-    Xobject bcastOpt = bcastDecl.getArg(3);
+    Xobject asyncOpt = bcastDecl.getArg(3);
 
-    if(bcastOpt != null){
-      XMP.fatal("bcast opt is not supported yet, sorry!");
-      return;
-    }
+    // if(bcastOpt != null){
+    //   XMP.fatal("bcast opt is not supported yet, sorry!");
+    //   return;
+    // }
 
     Vector<Ident> bcast_vars = new Vector<Ident>();
     for(Xobject v: bcastNameList){
@@ -667,18 +673,27 @@ public class XMPanalyzePragma
     info.setBcastInfo(XMPobjectsRef.parseDecl(fromRef,env,pb),
 		      XMPobjectsRef.parseDecl(onRef,env,pb),
 		      bcast_vars);
+
+    if (asyncOpt != null && !XmOption.isAsync()){
+      XMP.errorAt(pb, "MPI-3 is required to use the async clause on a bcast directive");
+    }
+
+    info.setAsyncId(asyncOpt);
   }
 
   private void analyzeWaitAsync(Xobject waitAsyncDecl, 
 				XMPinfo info, PragmaBlock pb){
+
     XobjList asyncIdList = (XobjList) waitAsyncDecl.getArg(0);
     Vector<Xobject> asyncIds = new Vector<Xobject>();
     for (Xobject x: asyncIdList){
 	asyncIds.add(x);
     }
     info.setWaitAsyncIds(asyncIds);
-    //    Xobject async_id = waitAsyncDecl.getArg(0);
-    //    info.setAsyncId(async_id);
+
+    Xobject onRef = waitAsyncDecl.getArg(1);
+    info.setOnRef(XMPobjectsRef.parseDecl(onRef, env, pb));
+
   }
 
   void analyzeTask(Xobject taskDecl, BlockList taskBody,
@@ -861,9 +876,12 @@ public class XMPanalyzePragma
       }
       else if (sub.Opcode() == Xcode.F_INDEX_RANGE){
 
+	int tidx = leftArray.getAlignSubscriptIndexAt(i);
+	if (tidx == -1) continue;
+
     	var = env.declIdent(XMP.genSym("XMP_loop_i"), Xtype.intType, pb);
     	varList.add(var);
-	varListTemplate.set(leftArray.getAlignSubscriptIndexAt(i), var);
+	varListTemplate.set(tidx, var);
 
 	lb = ((XobjList)sub).getArg(0);
 	if (lb == null){
@@ -927,6 +945,9 @@ public class XMPanalyzePragma
 	  Xobject sub = subscripts1.getArg(i);
 	  if (sub.Opcode() == Xcode.F_INDEX_RANGE){
 
+	    int tidx = array1.getAlignSubscriptIndexAt(i);
+	    if (tidx == -1) continue;
+
 	    Xobject lb, st;
 
 	    lb = ((XobjList)sub).getArg(0);
@@ -945,7 +966,7 @@ public class XMPanalyzePragma
 
 	    Xobject expr;
 	    //expr = Xcons.binaryOp(Xcode.MUL_EXPR, varList.get(k).Ref(), st);
-	    Ident loopVar = varListTemplate.get(array1.getAlignSubscriptIndexAt(i));
+	    Ident loopVar = varListTemplate.get(tidx);
 	    if (loopVar == null) XMP.fatal("array on rhs does not conform to that on lhs.");
 	    expr = Xcons.binaryOp(Xcode.MUL_EXPR, loopVar.Ref(), st);
 	    expr = Xcons.binaryOp(Xcode.PLUS_EXPR, expr, lb);
@@ -1002,7 +1023,7 @@ public class XMPanalyzePragma
 
     Xobject onSubscripts = pb.getClauses().getArg(0).getArg(1);
 
-    if (onSubscripts != null){
+    if (onSubscripts != null && !onSubscripts.isEmptyList()){
       int k = 0;
       for (int i = 0; i < onSubscripts.Nargs(); i++){
     	Xobject sub = onSubscripts.getArg(i);
