@@ -1,5 +1,5 @@
 /*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -57,6 +57,8 @@ public class XMParray {
 
   private int NthAssumedShape = -1;
 
+  boolean shadow_declared = false;
+
   // null constructor
   public XMParray() { }
 
@@ -286,6 +288,10 @@ public class XMParray {
     switch(sclass){
     case FPARAM: {
 
+      if (type.isFallocatable()){
+	XMP.errorAt(pb, "allocatable dummy arrays cannot be distributed.");
+      }
+
       Xtype ftype = env.getCurrentDef().getDef().getNameObj().Type();
 
       if (ftype.isFunction() && !ftype.isFsubroutine() && type.isFassumedShape()){
@@ -474,6 +480,13 @@ public class XMParray {
       XMP.errorAt(pb,"shadow dimension size is different from array dimension");
       return;
     }
+
+    if (array.shadow_declared){
+      XMP.errorAt(pb, "variable '" + name + "' already has shadow region");
+    }
+
+    array.shadow_declared = true;
+
     for(int i = 0; i < dims.size(); i++){
       XMPdimInfo d_info = dims.elementAt(i);
       int right = 0;
@@ -573,9 +586,12 @@ public class XMParray {
     for(int i = 0; i < dims.size(); i++){
       XMPdimInfo info = dims.elementAt(i);
 
+      Xobject lower = info.getLower();
       Xobject upper = null;
       if (info.getUpper() == null && NthAssumedShape >= 0 && NthAssumedShape < XMP.MAX_ASSUMED_SHAPE){
 	upper = Xcons.FarrayRef(sizeArray.Ref(), Xcons.IntConstant(NthAssumedShape), Xcons.IntConstant(i));
+	upper = Xcons.binaryOp(Xcode.PLUS_EXPR, upper, lower);
+	upper = Xcons.binaryOp(Xcode.MINUS_EXPR, upper, Xcons.IntConstant(1));
       }
       else {
 	upper = info.getUpper();
@@ -583,14 +599,14 @@ public class XMParray {
 
       if(info.isAlignAny()){
 	args = Xcons.List(descId.Ref(),Xcons.IntConstant(i),
-			  info.getLower(), upper,
+			  lower, upper,
 			  Xcons.IntConstant(-1),
 			  Xcons.IntConstant(0));
       } else {
 	Xobject off = info.getAlignSubscriptOffset();
 	if(off == null) off = Xcons.IntConstant(0);
 	args = Xcons.List(descId.Ref(),Xcons.IntConstant(i),
-			  info.getLower(), upper,
+			  lower, upper,
 			  Xcons.IntConstant(info.getAlignSubscriptIndex()),
 			  off);
       }
