@@ -1,5 +1,5 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -26,6 +26,7 @@ package exc.xmpF;
 import exc.object.*;
 import exc.block.*;
 import java.util.*;
+import static xcodeml.util.XmLog.fatal;
 
 /**
  * pass2: check and write variables
@@ -45,7 +46,6 @@ public class XMPrewriteExpr
     if (fb == null) return;
 
     // rewrite return statements
-
     BlockIterator iter5 = new topdownBlockIterator(fb);
     for (iter5.init(); !iter5.end(); iter5.next()){
       if (iter5.getBlock().Opcode() == Xcode.RETURN_STATEMENT){
@@ -55,7 +55,6 @@ public class XMPrewriteExpr
     }
 
     // rewrite allocate, deallocate, and stop statements
-
     BasicBlockIterator iter3 = new BasicBlockIterator(fb);
     for (iter3.init(); !iter3.end(); iter3.next()){
       StatementIterator iter4 = iter3.getBasicBlock().statements();
@@ -71,7 +70,12 @@ public class XMPrewriteExpr
 	    while (y.hasNext()){
 	      XobjList alloc = (XobjList)y.next();
 	      Xobject obj = alloc.getArg(0);
-	      while (obj.Opcode() == Xcode.MEMBER_REF) obj = obj.getArg(0).getArg(0);
+
+	      while (obj.Opcode() == Xcode.MEMBER_REF ||
+		     obj.Opcode() == Xcode.F_ARRAY_REF ||
+		     obj.Opcode() == Xcode.F_VAR_REF)
+		  obj = obj.getArg(0).getArg(0);
+
 	      Ident id = env.findVarIdent(obj.getName(), fb);
 	      if (id == null) break;
 	      XMParray array = XMParray.getArray(id);
@@ -88,7 +92,10 @@ public class XMPrewriteExpr
 	    while (y.hasNext()){
 	      XobjList dealloc = (XobjList)y.next();
 	      Xobject obj = dealloc.getArg(0);
-	      while (obj.Opcode() == Xcode.MEMBER_REF) obj = obj.getArg(0).getArg(0);
+	      while (obj.Opcode() == Xcode.MEMBER_REF ||
+		     obj.Opcode() == Xcode.F_ARRAY_REF ||
+		     obj.Opcode() == Xcode.F_VAR_REF)
+		obj = obj.getArg(0).getArg(0);
 	      Ident id = env.findVarIdent(obj.getName(), fb);
 	      if (id == null) break;
 	      XMParray array = XMParray.getArray(id);
@@ -185,6 +192,7 @@ public class XMPrewriteExpr
     for(iter.init(); !iter.end();iter.next()){
       Xobject x = iter.getXobject();
       if (x == null)  continue;
+      if (x.Opcode() == null) continue;      // #060  see [Xmp-dev:4675]
       switch (x.Opcode()) {
       case VAR:
 	{
@@ -212,8 +220,21 @@ public class XMPrewriteExpr
 	  if(array == null) break;
 
 	  int dim_i = 0;
+	  boolean no_leading_scalar_in_subscripts = true;
 	  for(XobjArgs args = x.getArg(1).getArgs(); args != null;
 	      args = args.nextArgs()){
+
+	    // check subscripts
+	    // if (array.isDistributed(dim_i) &&
+	    // 	args.getArg().Opcode() == Xcode.F_ARRAY_INDEX){
+	    //   no_leading_scalar_in_subscripts = false;
+	    // }
+	    // else if (array.isDistributed(dim_i) &&
+	    // 	     args.getArg().Opcode() == Xcode.F_INDEX_RANGE &&
+	    // 	     !no_leading_scalar_in_subscripts){
+	    //   XMP.errorAt(block, "':' must not be lead by any int-expr in a subscript list of a global array.");
+	    // }
+
 	    Xobject index_calc = 
 	      arrayIndexCalc(array,dim_i++,args.getArg(),bb,block);
 	    if(index_calc != null) args.setArg(index_calc);
@@ -228,9 +249,9 @@ public class XMPrewriteExpr
         // XXX delete this
       case CO_ARRAY_REF:
 	{
-	  System.out.println("coarray not yet: "+ x);
 	  break;
 	}
+
       case FUNCTION_CALL:
 	{
 	  String fname = x.getArg(0).getString();
@@ -587,7 +608,19 @@ public class XMPrewriteExpr
       i.setArg(0,x);
       return i;
 
-      /* what to do for array_range expression */
+    case F_INDEX_RANGE:
+      if (!a.isDistributed(dim_i)) return i;
+
+      if (is_colon(i, a, dim_i)){ // NOTE: this check is not strict.
+      	// if (a.hasShadow(dim_i) && dim_i != a.getDim() - 1){
+      	//   XMP.errorAt(block, "a subscript of the dimension having shadow must be an int-expr unless it is the last dimension.");
+      	// }
+	i.setArg(0, null); i.setArg(1, null); i.setArg(2, null);
+      	return i;
+      }
+
+      // what to do for other cases?
+
     default:
       XMP.errorAt(block,"bad expression in XMP array index");
       return null;
@@ -642,48 +675,126 @@ public class XMPrewriteExpr
       if (!param_list.getArg(i).Type().isFassumedShape()) continue;
 	
       Xobject arg = arg_list.getArg(i);
-      Ident id = env.findVarIdent(arg.getName(), fb);
-      XMParray array = XMParray.getArray(id);
 
-      Xtype atype;
-      int arrayDim;
-      Ident sizeFunc;
-      Xobject arg0;
+      if (arg.Opcode() == Xcode.VAR ||
+	  arg.Opcode() == Xcode.MEMBER_REF){ // just array name
+	  
+	Xtype atype;
+	int arrayDim;
+	Ident sizeFunc;
+	Xobject arg0;
+	XMParray array = null;
 
-      if (array != null){
-	atype = array.getType();
-	arrayDim = array.getDim();
-	//sizeFunc = env.declIntrinsicIdent("xmp_array_gsize", Xtype.FintFunctionType);
-	sizeFunc = env.declExternIdent("xmp_array_gsize", Xtype.FintFunctionType);
-	arg0 = array.getDescId().Ref();
-      }
-      else {
-	atype = arg.Type();
-	arrayDim = atype.getNumDimensions();
-	sizeFunc = env.declIntrinsicIdent("size", Xtype.FintFunctionType);
-	arg0 = arg;
-      }
-
-      if (atype.isFallocatable() || atype.isFassumedShape()){
-	for (int j = 0; j < arrayDim; j++){
-	  Xobject lhs = Xcons.FarrayRef(sizeArray.Ref(), Xcons.IntConstant(k), Xcons.IntConstant(j));
-	  Xobject rhs = sizeFunc.Call(Xcons.List(arg0, Xcons.IntConstant(j+1)));
-	  st.insert(Xcons.Set(lhs, rhs));
+	if (arg.Opcode() != Xcode.MEMBER_REF){
+	  Ident id = env.findVarIdent(arg.getName(), fb);
+	  array = XMParray.getArray(id);
 	}
+
+	if (array != null){
+	  atype = array.getType();
+	  arrayDim = array.getDim();
+	  //sizeFunc = env.declIntrinsicIdent("xmp_array_gsize", Xtype.FintFunctionType);
+	  sizeFunc = env.declExternIdent("xmp_array_gsize", Xtype.FintFunctionType);
+	  arg0 = array.getDescId().Ref();
+	}
+	else {
+	  atype = arg.Type();
+	  arrayDim = atype.getNumDimensions();
+	  sizeFunc = env.declIntrinsicIdent("size", Xtype.FintFunctionType);
+	  arg0 = arg;
+	}
+
+	if (atype.isFallocatable() || atype.isFassumedShape()){
+	  for (int j = 0; j < arrayDim; j++){
+	    Xobject lhs = Xcons.FarrayRef(sizeArray.Ref(), Xcons.IntConstant(k), Xcons.IntConstant(j));
+	    Xobject rhs = sizeFunc.Call(Xcons.List(arg0, Xcons.IntConstant(j+1)));
+	    st.insert(Xcons.Set(lhs, rhs));
+	  }
+	}
+	else {
+	  Xobject declSize[] = atype.getFarraySizeExpr();
+	  for (int j = 0; j < arrayDim; j++){
+	    Xobject lhs = Xcons.FarrayRef(sizeArray.Ref(), Xcons.IntConstant(k), Xcons.IntConstant(j));
+	    Xobject rhs = Xcons.binaryOp(Xcode.PLUS_EXPR,
+					 Xcons.binaryOp(Xcode.MINUS_EXPR,
+							declSize[j].getArg(1),
+							declSize[j].getArg(0)),
+					 Xcons.IntConstant(1));
+	    st.insert(Xcons.Set(lhs, rhs));
+	  }
+
+	}
+
       }
-      else {
-	Xobject declSize[] = atype.getFarraySizeExpr();
+      else if (arg.Opcode() == Xcode.F_ARRAY_REF){ // array section
+
+	Xtype atype;
+	int arrayDim;
+	Ident lbFunc, ubFunc;
+	Xobject arg0;
+	XMParray array = null;
+
+	if (arg.getArg(0).getArg(0).Opcode() != Xcode.MEMBER_REF){
+	  Ident id = env.findVarIdent(arg.getArg(0).getArg(0).getName(), fb);
+	  array = XMParray.getArray(id);
+	}
+
+	if (array != null){
+	  atype = array.getType();
+	  arrayDim = array.getDim();
+	  lbFunc = env.declExternIdent("xmp_array_lbound", Xtype.FintFunctionType);
+	  ubFunc = env.declExternIdent("xmp_array_ubound", Xtype.FintFunctionType);
+	  arg0 = array.getDescId().Ref();
+	}
+	else {
+	  atype = arg.getArg(0).getArg(0).Type();
+	  arrayDim = atype.getNumDimensions();
+	  lbFunc = env.declIntrinsicIdent("lbound", Xtype.FintFunctionType);
+	  ubFunc = env.declIntrinsicIdent("ubound", Xtype.FintFunctionType);
+	  arg0 = arg.getArg(0).getArg(0);
+	}
+
+	XobjList subList = (XobjList)arg.getArg(1);
 	for (int j = 0; j < arrayDim; j++){
+
+	  Xobject sub = subList.getArg(j);
+	  Xobject rhs = null;
+
+	  if (sub.Opcode() == Xcode.F_ARRAY_INDEX){
+	    //rhs = Xcons.IntConstant(1);
+	    continue;
+	  }
+	  else if (sub.Opcode() == Xcode.F_INDEX_RANGE){
+	    Xobject lb = sub.getArg(0);
+	    if (lb == null){
+	      lb = lbFunc.Call(Xcons.List(arg0, Xcons.IntConstant(j+1)));
+	    }
+	    Xobject ub = sub.getArg(1);
+	    if (ub == null){
+	      ub = ubFunc.Call(Xcons.List(arg0, Xcons.IntConstant(j+1)));
+	    }
+
+	    Xobject stride = sub.getArg(2);
+	    if (stride == null){
+	      rhs = Xcons.binaryOp(Xcode.PLUS_EXPR,
+				   Xcons.binaryOp(Xcode.MINUS_EXPR, ub, lb),
+				   Xcons.IntConstant(1));
+	    }
+	    else {
+	      rhs = Xcons.binaryOp(Xcode.MINUS_EXPR, ub, lb);
+	      rhs = Xcons.binaryOp(Xcode.PLUS_EXPR, rhs, stride);
+	      rhs = Xcons.binaryOp(Xcode.DIV_EXPR, rhs, stride);
+	    }
+	  }
+
 	  Xobject lhs = Xcons.FarrayRef(sizeArray.Ref(), Xcons.IntConstant(k), Xcons.IntConstant(j));
-	  Xobject rhs = Xcons.binaryOp(Xcode.PLUS_EXPR,
-				       Xcons.binaryOp(Xcode.MINUS_EXPR,
-						      declSize[j].getArg(1),
-						      declSize[j].getArg(0)),
-				       Xcons.IntConstant(1));
 	  st.insert(Xcons.Set(lhs, rhs));
 	}
 
       }
+      else {
+	continue;
+      }
 
       k++;
       if (k > XMP.MAX_ASSUMED_SHAPE){
@@ -694,4 +805,28 @@ public class XMPrewriteExpr
 
   }
 
+  private boolean is_colon(Xobject i, XMParray a, int dim_i){
+
+    // check lower
+    if (i.getArg(0) != null){
+      Xobject lb = a.getType().getFarraySizeExpr()[dim_i].getArg(0);
+      if (!lb.equals(i.getArg(0))) return false;
+    }
+
+    // check upper
+    if (i.getArg(1) != null){
+      Xobject ub = a.getType().getFarraySizeExpr()[dim_i].getArg(1);
+      if (!ub.equals(i.getArg(1))) return false;
+    }
+
+    // check stride
+    if (i.getArg(2) != null){
+      if (i.getArg(2).Opcode() != Xcode.INT_CONSTANT ||
+  	  i.getArg(2).getInt() != 1) return false;
+    }
+
+    return true;
+
+  }
+
 }
