@@ -1,5 +1,5 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -79,20 +79,46 @@ public class XMPtransPragma
     }
 
     if(env.currentDefIsModule()){
+
       Xtype save_logical = Xtype.FlogicalType.copy();
       save_logical.setIsFsave(true);
-      save_logical.setIsFprivate(true);
-      Ident init_flag_var = env.declIdent("xmpf_init_flag",save_logical);
+      //save_logical.setIsFprivate(true);
+      //Ident init_flag_var = env.declIdent("xmpf_init_flag", save_logical);
+      BlockList body= env.getCurrentDef().getBlock().getBody();
+      Ident init_flag_var = body.declLocalIdent("xmpf_init_flag_"+env.currentDefName(), save_logical,
+      						StorageClass.FSAVE,
+      						Xcons.List(Xcode.F_VALUE, Xcons.FlogicalConstant(false)));
+
+      // NOTE: it is guaranteed that no threadprivate exists.
+
+      Vector<XMPmodule> modules = env.getModules();
+      for (int m = modules.size() - 1; m >= 0; m--){
+	Ident f = env.declIdent("xmpf_traverse_module_"+modules.get(m).getModuleName(),
+				Xtype.FsubroutineType);
+	prolog.insert(f.callSubroutine(Xcons.List()));
+      }
+
+      prolog.insert(Bcons.IF(init_flag_var.Ref(), Xcons.List(Xcode.RETURN_STATEMENT), null));
       prolog.add(Xcons.Set(init_flag_var.Ref(),
 			   Xcons.FlogicalConstant(true)));
 
+      // Ident prolog_f = 
+      // 	env.declIdent(env.currentDefName()+"_xmpf_module_init_",
+      // 		      Xtype.FsubroutineType);
+      // XobjectDef prolog_def = 
+      // 	XobjectDef.Func(prolog_f, null, null, prolog.toXobject());
+      // env.getCurrentDef().getDef().getChildren().addFirst(prolog_def);
+      // prolog_def.setParent(env.getCurrentDef().getDef());
+
       Ident prolog_f = 
-	env.declIdent(env.currentDefName()+"_xmpf_module_init_",
+      	env.declIdent("xmpf_traverse_module_"+env.currentDefName(),
 		      Xtype.FsubroutineType);
+      XobjString modName = Xcons.Symbol(Xcode.IDENT, env.getCurrentDef().getDef().getName());
+      Xobject decls = Xcons.List(Xcons.List(Xcode.F_USE_DECL, modName));
       XobjectDef prolog_def = 
-	XobjectDef.Func(prolog_f, null, null, prolog.toXobject());
-      env.getCurrentDef().getDef().getChildren().addFirst(prolog_def);
-      prolog_def.setParent(env.getCurrentDef().getDef());
+      	XobjectDef.Func(prolog_f, null, decls, prolog.toXobject());
+      env.getEnv().add(prolog_def);
+
     } else {
       // fblock = (FunckBlock ident <param_env> [BlockList 
       //   (CompoundBlock <local_env> [BlockList statment ...]))
@@ -405,6 +431,12 @@ public class XMPtransPragma
 				    Xcons.Cast(Xtype.voidPtrType, Xcons.IntConstant(0)));
     }
 
+    if (info.getAsyncId() != null){
+      Xobject arg = Xcons.List(info.getAsyncId());
+      Ident g = env.declInternIdent(XMP.init_async_f, Xtype.FsubroutineType);
+      ret_body.add(g.callSubroutine(arg));
+    }
+
     if (on_ref != null){
       if (info.pragma != XMPpragma.LOOP){
 	ret_body.add(on_ref.buildConstructor(env));
@@ -420,8 +452,13 @@ public class XMPtransPragma
 
     // object size
     int op = info.getReductionOp();
-    Ident f = env.declInternIdent(XMP.reduction_f,
-				  Xtype.FsubroutineType);
+    // boolean reduce_minus = false;
+    // if (op == XMP.REDUCE_MINUS){
+    //   op = XMP.REDUCE_SUM;
+    //   reduce_minus = true;
+    // }
+
+    Ident f = env.declInternIdent(XMP.reduction_f, Xtype.FsubroutineType);
     Ident f2 = env.declInternIdent(XMP.reduction_loc_f, Xtype.FsubroutineType);
 
     //for(Ident id: info.getReductionVars()){
@@ -457,9 +494,20 @@ public class XMPtransPragma
 	ret_body.add(f2.callSubroutine(args2));
 	j++;
       }
-	  
+
+      // if (reduce_minus){
+      // 	ret_body.add(Xcons.Set(id.Ref(), Xcons.unaryOp(Xcode.UNARY_MINUS_EXPR, id.Ref())));
+      // }
+
       ret_body.add(f.callSubroutine(args));
     }
+
+    if (info.getAsyncId() != null){
+      Xobject arg = Xcons.List(info.getAsyncId());
+      Ident g = env.declInternIdent(XMP.start_async_f, Xtype.FsubroutineType);
+      ret_body.add(g.callSubroutine(arg));
+    }
+
     return Bcons.COMPOUND(ret_body);
   }
 
@@ -476,6 +524,12 @@ public class XMPtransPragma
 				    Xcons.Cast(Xtype.voidPtrType, Xcons.IntConstant(0)));
     }
 
+    if (info.getAsyncId() != null){
+      Xobject arg = Xcons.List(info.getAsyncId());
+      Ident g = env.declInternIdent(XMP.init_async_f, Xtype.FsubroutineType);
+      ret_body.add(g.callSubroutine(arg));
+    }
+
     XMPobjectsRef from_ref = info.getBcastFrom();
     Xobject from_ref_arg;
     if (from_ref != null){
@@ -492,8 +546,8 @@ public class XMPtransPragma
     }
     else on_ref_arg = xmp_null;
 
-    Ident f = env.declInternIdent(XMP.bcast_f,
-				  Xtype.FsubroutineType);
+    Ident f = env.declInternIdent(XMP.bcast_f, Xtype.FsubroutineType);
+
     for(Ident id: info.getInfoVarIdents()){
       Xtype type = id.Type();
       Xobject size_expr = Xcons.IntConstant(1);
@@ -534,28 +588,46 @@ public class XMPtransPragma
       if(!type.isBasic()){
 	XMP.fatal("bcast for non-basic type ="+type);
       }
-      Xobject args = Xcons.List(id.Ref(),size_expr,
+
+      Xobject args = Xcons.List(id.Ref(), size_expr,
 				XMP.typeIntConstant(type),
 				from_ref_arg,
 				on_ref_arg);
+
       ret_body.add(f.callSubroutine(args));
+
     }
+
+    if (info.getAsyncId() != null){
+      Xobject arg = Xcons.List(info.getAsyncId());
+      Ident g = env.declInternIdent(XMP.start_async_f, Xtype.FsubroutineType);
+      ret_body.add(g.callSubroutine(arg));
+    }
+
     return Bcons.COMPOUND(ret_body);
   }
 
   private Block translateWaitAsync(PragmaBlock pb, XMPinfo info){
-    Block b = Bcons.emptyBlock();
-    BasicBlock bb = b.getBasicBlock();
+    //Block b = Bcons.emptyBlock();
+    //BasicBlock bb = b.getBasicBlock();
+
+    BlockList ret_body = Bcons.emptyBody();
+
+    XMPobjectsRef on_ref = info.getOnRef();
+    Xobject on_ref_arg;
+    if (on_ref != null){
+      ret_body.add(on_ref.buildConstructor(env));
+      on_ref_arg = on_ref.getDescId().Ref();
+    }
+    else on_ref_arg = env.getNullIdent(pb).Ref();
 
     Ident f = env.declInternIdent(XMP.wait_async_f, Xtype.FsubroutineType);
 
     for (Xobject i: info.waitAsyncIds){
-	bb.add(f.callSubroutine(Xcons.List(i)));
+      ret_body.add(f.callSubroutine(Xcons.List(i, on_ref_arg)));
     }
 
-    //    bb.add(f.callSubroutine(Xcons.List(info.async_id)));
-
-    return b;
+    return Bcons.COMPOUND(ret_body);
   }
 
   private Block translateTask(PragmaBlock pb, XMPinfo info){
