@@ -1,5 +1,5 @@
 /*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
+ * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
  * $TSUKUBA_Copyright:
  *  Copyright (C) 2010-2014 University of Tsukuba, 
  *  	      2012-2014  University of Tsukuba and Riken AICS
@@ -21,6 +21,9 @@
  *    Culture, Sports, Science and Technology, Japan.
  *  $
  */
+#ifndef MPI_PORTABLE_PLATFORM_H
+#define MPI_PORTABLE_PLATFORM_H
+#endif 
 
 #include <stdarg.h>
 #include "mpi.h"
@@ -971,7 +974,7 @@ int _XMP_calc_linear_rank_on_target_nodes(_XMP_nodes_t *n, int *rank_array,
 }
 
 _XMP_nodes_ref_t *_XMP_init_nodes_ref(_XMP_nodes_t *n, int *rank_array) {
-  _XMP_nodes_ref_t *nodes_ref = _XMP_alloc(sizeof(sizeof(int)));
+  _XMP_nodes_ref_t *nodes_ref = _XMP_alloc(sizeof(_XMP_nodes_ref_t));
   int dim = n->dim;
   int *new_rank_array = _XMP_alloc(sizeof(int) * dim);
 
