@@ -1,9 +1,12 @@
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include "xmp_internal.h"
 #include "mpi.h"
-#ifdef _XMP_COARRAY_FJRDMA
+
+#ifdef _XMP_FJRDMA
 #include "mpi-ext.h"
 #endif
-#include <stdio.h>
 
 static int _XMP_runtime_working = _XMP_N_INT_FALSE;
 int _XMPC_running = 1;
@@ -12,11 +15,13 @@ int _XMPF_running = 0;
 void _XMP_init(int argc, char** argv)
 {
   if (!_XMP_runtime_working) {
-#if defined(_XMP_COARRAY_GASNET) || defined(_XMP_COARRAY_FJRDMA)
-    _XMP_coarray_initialize(argc, argv);
-    _XMP_post_wait_initialize();
-#endif
+    MPI_Init(&argc, &argv);
+    MPI_Comm_rank(MPI_COMM_WORLD, &_XMP_world_rank);
+    MPI_Comm_size(MPI_COMM_WORLD, &_XMP_world_size);
 
+#if defined(_XMP_GASNET) || defined(_XMP_FJRDMA)
+    _XMP_onesided_initialize(argc, argv);
+#endif
 #ifdef _XMP_TCA
     _XMP_init_tca();
 #endif
@@ -29,10 +34,8 @@ void _XMP_init(int argc, char** argv)
 void _XMP_finalize(int return_val)
 {
   if (_XMP_runtime_working) {
-#ifdef _XMP_COARRAY_GASNET
-    _XMP_coarray_finalize(return_val);
-#elif _XMP_COARRAY_FJRDMA
-    _XMP_fjrdma_finalize();
+#if defined(_XMP_GASNET) || defined(_XMP_FJRDMA)
+    _XMP_onesided_finalize(return_val);
 #endif
     _XMP_finalize_world();
     _XMP_runtime_working = _XMP_N_INT_FALSE;
@@ -53,3 +56,4 @@ void xmpc_finalize_all(int return_val)
 {
   _XMP_finalize(return_val);
 }
+
