@@ -13,12 +13,6 @@ void xmpf_reflect__(_XMP_array_t **a_desc)
 }
 
 
-void xmpf_wait_async__(int *async_id)
-{
-  _XMP_wait_async__(*async_id);
-}
-
-
 void xmpf_reflect_async__(_XMP_array_t **a_desc, int *async_id)
 {
   _XMP_reflect_async__(*a_desc, *async_id);
