@@ -3,6 +3,7 @@
 
 extern int chk_int(int ierr);
 extern int chk_int2(int ierr);
+extern void xmp_matmul(void *x_p, void *a_p, void *b_p);
 
 #pragma xmp nodes p(4)
 
@@ -21,7 +22,7 @@ double a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -85,7 +86,7 @@ double a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -149,7 +150,7 @@ double _Complex a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -213,7 +214,7 @@ float _Complex a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -277,7 +278,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -340,7 +341,7 @@ float a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -404,7 +405,7 @@ double a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(j,i)
 #pragma xmp align x[i][j] with tz(j,i)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(j,i)
@@ -468,7 +469,7 @@ double a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -533,7 +534,7 @@ double a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -598,7 +599,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[*][j] with tz(j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -666,7 +667,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][*] with tz(i)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -710,7 +711,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on q
+#pragma xmp task on q
 {
   chk_int2(ierr);
 }
@@ -734,7 +735,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -799,7 +800,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(j)
 #pragma xmp align x[*][j] with tz(j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -843,7 +844,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on q
+#pragma xmp task on q
 {
   chk_int2(ierr);
 }
@@ -867,7 +868,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(j)
 #pragma xmp align x[i][*] with tz(i)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -911,7 +912,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on q
+#pragma xmp task on q
 {
   chk_int2(ierr);
 }
@@ -935,7 +936,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -1000,7 +1001,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i)
 #pragma xmp align x[*][j] with tz(j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -1044,7 +1045,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on q
+#pragma xmp task on q
 {
   chk_int2(ierr);
 }
@@ -1068,7 +1069,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i)
 #pragma xmp align x[i][*] with tz(i)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -1112,7 +1113,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on q
+#pragma xmp task on q
 {
   chk_int2(ierr);
 }
@@ -1136,7 +1137,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -1201,7 +1202,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[*][j] with tz(j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -1245,7 +1246,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on q
+#pragma xmp task on q
 {
   chk_int2(ierr);
 }
@@ -1269,7 +1270,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][*] with tz(i)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -1313,7 +1314,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on q
+#pragma xmp task on q
 {
   chk_int2(ierr);
 }
@@ -1337,7 +1338,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -1401,7 +1402,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(j)
 #pragma xmp align x[*][j] with tz(j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -1465,7 +1466,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(j)
 #pragma xmp align x[i][*] with tz(i)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -1530,7 +1531,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -1594,7 +1595,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i)
 #pragma xmp align x[*][j] with tz(j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -1658,7 +1659,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i)
 #pragma xmp align x[i][*] with tz(i)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -1723,7 +1724,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i)
@@ -1788,7 +1789,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[*][j] with tz(j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i)
@@ -1832,7 +1833,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on q
+#pragma xmp task on q
 {
   chk_int2(ierr);
 }
@@ -1856,7 +1857,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][*] with tz(i)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i)
@@ -1900,7 +1901,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on q
+#pragma xmp task on q
 {
   chk_int2(ierr);
 }
@@ -1924,7 +1925,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i)
@@ -1988,7 +1989,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(j)
 #pragma xmp align x[*][j] with tz(j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i)
@@ -2052,7 +2053,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(j)
 #pragma xmp align x[i][*] with tz(i)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i)
@@ -2117,7 +2118,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i)
@@ -2181,7 +2182,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i)
 #pragma xmp align x[*][j] with tz(j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i)
@@ -2245,7 +2246,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i)
 #pragma xmp align x[i][*] with tz(i)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i)
