@@ -3,7 +3,7 @@
 
 extern int chk_int(int ierr);
 extern int chk_int2(int ierr);
-
+extern void xmp_matmul(void *x_p, void *a_p, void *b_p);
 #pragma xmp nodes p(4)
 
 void test_mm_aani_b_c_bc_i4_d(){
@@ -21,7 +21,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[*][j] with tz(*,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -65,7 +65,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1,1:2)
+#pragma xmp task on p(1,1:2)
 {
   chk_int2(ierr);
 }
@@ -88,7 +88,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][*] with tz(i,*)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -132,7 +132,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1:2,1)
+#pragma xmp task on p(1:2,1)
 {
   chk_int2(ierr);
 }
@@ -155,7 +155,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(*,j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -219,7 +219,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(*,j)
 #pragma xmp align x[*][j] with tz(*,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -263,7 +263,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1,1:2)
+#pragma xmp task on p(1,1:2)
 {
   chk_int2(ierr);
 }
@@ -286,7 +286,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(*,j)
 #pragma xmp align x[i][*] with tz(i,*)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -330,7 +330,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1:2,1)
+#pragma xmp task on p(1:2,1)
 {
   chk_int2(ierr);
 }
@@ -353,7 +353,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i,*)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -417,7 +417,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i,*)
 #pragma xmp align x[*][j] with tz(*,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -461,7 +461,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1,1:2)
+#pragma xmp task on p(1,1:2)
 {
   chk_int2(ierr);
 }
@@ -484,7 +484,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i,*)
 #pragma xmp align x[i][*] with tz(i,*)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i,j) on tx(i,j)
@@ -528,7 +528,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1:2,1)
+#pragma xmp task on p(1:2,1)
 {
   chk_int2(ierr);
 }
@@ -551,7 +551,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -615,7 +615,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[*][j] with tz(*,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -659,7 +659,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1,1:2)
+#pragma xmp task on p(1,1:2)
 {
   chk_int2(ierr);
 }
@@ -682,7 +682,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][*] with tz(i,*)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -726,7 +726,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1:2,1)
+#pragma xmp task on p(1:2,1)
 {
   chk_int2(ierr);
 }
@@ -749,7 +749,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(*,j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -813,7 +813,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(*,j)
 #pragma xmp align x[*][j] with tz(*,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -857,7 +857,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1,1:2)
+#pragma xmp task on p(1,1:2)
 {
   chk_int2(ierr);
 }
@@ -880,7 +880,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(*,j)
 #pragma xmp align x[i][*] with tz(i,*)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -924,7 +924,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1:2,1)
+#pragma xmp task on p(1:2,1)
 {
   chk_int2(ierr);
 }
@@ -947,7 +947,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i,*)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -1011,7 +1011,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i,*)
 #pragma xmp align x[*][j] with tz(*,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -1055,7 +1055,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1,1:2)
+#pragma xmp task on p(1,1:2)
 {
   chk_int2(ierr);
 }
@@ -1078,7 +1078,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i,*)
 #pragma xmp align x[i][*] with tz(i,*)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
   for(i=0;i<n1;i++){
@@ -1122,7 +1122,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1:2,1)
+#pragma xmp task on p(1:2,1)
 {
   chk_int2(ierr);
 }
@@ -1145,7 +1145,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i,*)
@@ -1209,7 +1209,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[*][j] with tz(*,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i,*)
@@ -1253,7 +1253,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1,1:2)
+#pragma xmp task on p(1,1:2)
 {
   chk_int2(ierr);
 }
@@ -1276,7 +1276,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][j] with ty(i,j)
 #pragma xmp align x[i][*] with tz(i,*)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i,*)
@@ -1320,7 +1320,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1:2,1)
+#pragma xmp task on p(1:2,1)
 {
   chk_int2(ierr);
 }
@@ -1343,7 +1343,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(*,j)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i,*)
@@ -1407,7 +1407,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(*,j)
 #pragma xmp align x[*][j] with tz(*,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i,*)
@@ -1451,7 +1451,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1,1:2)
+#pragma xmp task on p(1,1:2)
 {
   chk_int2(ierr);
 }
@@ -1474,7 +1474,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[*][j] with ty(*,j)
 #pragma xmp align x[i][*] with tz(i,*)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i,*)
@@ -1518,7 +1518,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1:2,1)
+#pragma xmp task on p(1:2,1)
 {
   chk_int2(ierr);
 }
@@ -1541,7 +1541,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i,*)
 #pragma xmp align x[i][j] with tz(i,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i,*)
@@ -1605,7 +1605,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i,*)
 #pragma xmp align x[*][j] with tz(*,j)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i,*)
@@ -1649,7 +1649,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1,1:2)
+#pragma xmp task on p(1,1:2)
 {
   chk_int2(ierr);
 }
@@ -1672,7 +1672,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
 #pragma xmp align b[i][*] with ty(i,*)
 #pragma xmp align x[i][*] with tz(i,*)
 
-  int i,j,k,ierr;
+ int i,j,/*k,*/ierr;
   double rn1,rn2,rn3,rn4,rn5,rn6,ra,rb;
 
 #pragma xmp loop (i) on tx(i,*)
@@ -1716,7 +1716,7 @@ int a[n1][n2],b[n2][n3],x[n1][n3];
     }
   }
 
-#pragma task on p(1:2,1)
+#pragma xmp task on p(1:2,1)
 {
   chk_int2(ierr);
 }
